using System;
using System.Drawing.Design;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports fields property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class FldControlUtil : ControlUtil {
	    /// <summary>Common interface for controls attached to a field</summary>
	    public interface IFldControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Field name</summary>
	        FldFullName                 FieldFullName {  get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>Field attached to the control</summary>
	    protected Field                 m_fld;
	    /// <summary>Full name of the field</summary>
	    protected FldFullName           m_fldFullName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public FldControlUtil(object parent, bool bResDispenserOnly) : base(parent, bResDispenserOnly) {
		    m_fld           = null;
		    FieldFullName   = new FldFullName();
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:          Full name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public FldFullName FieldFullName {
		    get {
		        return(m_fldFullName);
		    }
		    set {
		        m_fldFullName   = value;
		        m_fld           = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Field:                  Rep++ Field attached to the control
		/// </summary>
        //
        //********************************************************************************
		public Field Field {
		    get {
                RepPP.Field         fldRetVal = null;
		        RepPP.Application   app;
		        
		        if (m_fld != null && m_fld.IsDetached) {
		            m_fld = null;
		        }
		        if (m_fld != null) {
                    fldRetVal = m_fld;
                } else {
		            app       = Application;
	                fldRetVal = app != null ? app.Field(FieldFullName.GroupName, FieldFullName.FieldName) : null;
                    if (!IsDesignMode) {
                        m_fld = fldRetVal;
                    }
		        }
		        return(fldRetVal);
		    }
		    set {
		        if (value != m_fld) {
		            m_fld = null;
		            if (value == null) {
		                FieldFullName = new FldFullName();
		            } else {
		                FieldFullName = new FldFullName(value.Group.Name, value.Name);
		                m_fld         = value;
		            }
		        }
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the FieldFullName property
	    /// </summary>
        //
        //********************************************************************************
        public class FldControlEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.Modal;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FormFldControlPropPage          frmFldControlPropPage;
                FldControlUtil.IFldControl      fldControl;
                IWindowsFormsEditorService      edSvc;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    if (edSvc != null) {
                        fldControl            = context.Instance as FldControlUtil.IFldControl;
                        frmFldControlPropPage = new FormFldControlPropPage(fldControl);
                        try {
                            edSvc.ShowDialog(frmFldControlPropPage);
                            objRetVal = fldControl.FieldFullName;
                        } finally {
                            frmFldControlPropPage.Dispose();
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class FldControlEditor

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement a popup listbox to be used to display the list of group or field
	    ///  for edition in a drop-down
	    /// </summary>
        //
        //********************************************************************************
        internal class PopupListBox : System.Windows.Forms.ListBox {
            /// <summary>Service which handle the dropdown</summary>
            private IWindowsFormsEditorService      m_edSvc;

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor
            /// </summary>
            /// <param name="edSvc">    Service hanling the dropdown</param>
            //
            //********************************************************************************
            public PopupListBox(IWindowsFormsEditorService edSvc) {
                m_edSvc = edSvc;
            }

            //********************************************************************************
            //
            /// <summary>
            /// OnSelectedIndexChanged:     Call when the selected item changed
            /// </summary>
            /// <param name="e">            Event parameter</param>
            //
            //********************************************************************************
            protected override void OnSelectedIndexChanged(EventArgs e) {
                base.OnSelectedIndexChanged (e);
                m_edSvc.CloseDropDown();
            }
        } // Class PopupListBox

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Group name property
	    /// </summary>
        //
        //********************************************************************************
        public class GroupNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FldFullName                     fldFullName;
                IWindowsFormsEditorService      edSvc;
                RepPP.Application               app;
                PopupListBox                    listBox;
                string                          strGrpName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    app   = RepPP.Application.GetApplicationForDesign(provider);
                    if (edSvc != null && app != null) {
                        fldFullName             = context.Instance as FldFullName;
                        objRetVal               = fldFullName.GroupName;
                        listBox                 = new PopupListBox(edSvc);
                        listBox.Location        = new System.Drawing.Point(0, 0);
                        listBox.Name            = "listBox";
                        listBox.Size            = new System.Drawing.Size(168, 186);
                        listBox.TabIndex        = 1;
                        listBox.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                        foreach (Group grp in app.Groups) {
                            listBox.Items.Add(grp.Name);
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strGrpName = listBox.SelectedItem as string;
                            objRetVal  = strGrpName;
                            if (strGrpName != fldFullName.GroupName) {
                                fldFullName.FieldName = "";
                            }
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class GroupNameEditor

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Field name property
	    /// </summary>
        //
        //********************************************************************************
        public class FieldNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FldFullName                     fldFullName;
                IWindowsFormsEditorService      edSvc;
                RepPP.Application               app;
                PopupListBox                    listBox;
                Group                           grp;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    app   = RepPP.Application.GetApplicationForDesign(provider);
                    if (edSvc != null && app != null) {
                        fldFullName             = context.Instance as FldFullName;
                        objRetVal               = fldFullName.GroupName;
                        listBox                 = new PopupListBox(edSvc);
                        listBox.Location        = new System.Drawing.Point(0, 0);
                        listBox.Name            = "listBox";
                        listBox.Size            = new System.Drawing.Size(168, 186);
                        listBox.TabIndex        = 1;
                        listBox.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                        grp                     = (fldFullName.GroupName != String.Empty) ? app.Groups[fldFullName.GroupName] : null;
                        if (grp != null) {
                            foreach (Field fld in grp.Fields) {
                                listBox.Items.Add(fld.Name);
                            }
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            objRetVal = listBox.SelectedItem as string;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class FieldNameEditor        
    } // Class FldControlUtil
} // Namespace