using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
	/// <summary>
	/// Handle two ListViewGInst listview use to pickup
	/// a list of item.
	/// </summary>
	//
    //*********************************************************
	public class DoubleSelectList : IDisposable
	{
	    /// <summary>
	    /// Listview of available items
	    /// </summary>
	    private ListViewGInst           m_lvAvailable;
	    /// <summary>
	    /// Listview of selected items
	    /// </summary>
	    private ListViewGInst           m_lvSelected;
	    /// <summary>
	    /// Add an item to the selected items
	    /// </summary>
	    private Button                  m_butAdd;
	    /// <summary>
	    /// Remove an item from the selected items
	    /// </summary>
	    private Button                  m_butRemove;
	    /// <summary>
	    /// true to remove item physically from available list
	    /// </summary>
	    private bool                    m_bAvPhysRemoved;
	    /// <summary>
	    /// true to remove item physically from selected list
	    /// </summary>
	    private bool                    m_bSelPhysRemoved;

        //*********************************************************
        //
		/// <summary>
		/// Class constructor
		/// </summary>
		/// <param name="lvAvailable">      Listview containing the list of available items</param>
		/// <param name="bAvPhysRemoved">   false = logical remove (mark as deleted), true = physically removed</param>
		/// <param name="lvSelected">       Listview containing the list of selected items</param>
		/// <param name="bSelPhysRemoved">  false = logical remove (mark as deleted), true = physically removed</param>
		/// <param name="butAdd">           Button use to add items to the selected one</param>
		/// <param name="butRemove">        Button use to remove items from the selected one</param>
		//
        //*********************************************************
		public DoubleSelectList(ListViewGInst     lvAvailable,
		                        bool              bAvPhysRemoved,
		                        ListViewGInst     lvSelected,
		                        bool              bSelPhysRemoved,
		                        Button            butAdd,
		                        Button            butRemove)
		{
		    m_lvAvailable     = lvAvailable;
		    m_bAvPhysRemoved  = bAvPhysRemoved;
		    m_lvSelected      = lvSelected;
		    m_bSelPhysRemoved = bSelPhysRemoved;
		    m_butAdd          = butAdd;
		    m_butRemove       = butRemove;
		    if (m_butAdd != null) {
		        m_butAdd.Click  += new EventHandler(OnAddClick);
		        m_butAdd.Enabled = (m_lvAvailable.SelectedItems.Count != 0);
		    }
		    if (m_butRemove != null) {
		        m_butRemove.Click  += new EventHandler(OnRemoveClick);
		        m_butRemove.Enabled = (m_lvSelected.SelectedItems.Count != 0);
		    }
            if (m_lvAvailable != null) {
		        m_lvAvailable.SelectedIndexChanged += new EventHandler(OnAvailableSelIndexChanged);
                m_lvAvailable.DoubleClick          += new EventHandler(OnAddClick);
            }
            if (m_lvSelected != null) {
		        m_lvSelected.SelectedIndexChanged  += new EventHandler(OnSelectedSelIndexChanged);
                m_lvSelected.DoubleClick           += new EventHandler(OnRemoveClick);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose:            Dispose the object
        /// </summary>
        //
        //*********************************************************
        public void Dispose()
        {
            if (m_butAdd != null) {
                m_butAdd.Click -= new EventHandler(OnAddClick);
            }
            if (m_butRemove != null) {
                m_butRemove.Click -= new EventHandler(OnRemoveClick);
            }
            if (m_lvAvailable != null) {
                m_lvAvailable.SelectedIndexChanged -= new EventHandler(OnAvailableSelIndexChanged);
                m_lvAvailable.DoubleClick          -= new EventHandler(OnAddClick);
            }
            if (m_lvSelected != null) {
		        m_lvSelected.SelectedIndexChanged  -= new EventHandler(OnSelectedSelIndexChanged);
                m_lvSelected.DoubleClick -= new EventHandler(OnRemoveClick);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetMaxFieldValuei:          Get the maximum value of a field in the group instance
        /// </summary>
        /// <param name="ginst">        The group instance </param>
        /// <param name="fld">          The field. It should be numeric </param>
        /// <returns>
        /// The maximum value of the field
        /// </returns>
        //
        //******************************************************************
        public static int GetMaxFieldValuei(RepPP.GroupInstance ginst, RepPP.Field fld)
        {
            int         iRetVal = 0;
            int         iIndex;
            int         iValue;

            ginst.SaveLinePos();
            for (iIndex = 0; iIndex < ginst.LineCount; iIndex++) {
                ginst.SelectLine(iIndex, false, false);
                iValue = Convert.ToInt32(fld.Value);
                if ( iValue > iRetVal) {
                    iRetVal = iValue;
                }
            }
            ginst.RestoreLinePos(false);
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// CopyItemsToListView:    Copy items to a listview
        /// </summary>
        /// <param name="lvSrc">    Source Listview</param>
        /// <param name="lvDest">   Destination listview</param>
        /// <param name="items">    List of items to add</param>
        //
        //*********************************************************
        private void CopyItemsToListView(ListViewGInst lvSrc, ListViewGInst lvDest, ListViewItem[] items)
        {
            RepPP.GroupInstance ginstSrc;
            RepPP.GroupInstance ginstDest;
            ListViewLineItem   gitem;
            int                 iLineIndex;
            int                 iPos = 0;
            
            ginstSrc    = lvSrc.GroupInstance;
            ginstDest   = lvDest.GroupInstance;
            if (lvDest.AutoRenumField != null) {
                iPos = GetMaxFieldValuei(ginstDest, lvDest.AutoRenumField);
            }
            foreach (ListViewItem item in items) {
                gitem = item as ListViewLineItem;
                if (gitem != null) {
                    iLineIndex = ginstSrc.LineFromBookmark(gitem.LineBookmark);
                    if (iLineIndex != -1) {
                        ginstSrc.SelectLine(iLineIndex, false, false);
                        ginstDest.AddBlankLine();
                        ginstDest.CopyLineFrom(ginstSrc, RepPP.CopyFieldOpt.sdCopyState, false, false);
                        if (lvDest.AutoRenumField != null) {
                            iPos                        += 10;
                            lvDest.AutoRenumField.Value  = Convert.ToString(iPos);
                        }
                        lvDest.InsertItem(-1, true);
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RemoveItemsToListView:      Remove items from a listview
        /// </summary>
        /// <param name="lv">           Listview</param>
        /// <param name="bPhysRemoved"> true to physicaly removed the item from group instance</param>
        /// <param name="items">        List of items to add</param>
        //
        //*********************************************************
        private void RemoveItemsFromListView(ListViewGInst lv, bool bPhysRemoved, ListViewItem[] items)
        {
            RepPP.GroupInstance ginst;
            int                 iLineIndex;
            ListViewLineItem    gitem;
            
            ginst = lv.GroupInstance;
            foreach (ListViewItem item in items) {
                gitem = item as ListViewLineItem;
                if (gitem != null) {
                    iLineIndex = ginst.LineFromBookmark(gitem.LineBookmark);
                    if (iLineIndex != -1) {
                        ginst.SelectLine(iLineIndex, false, false);                        
                        if (bPhysRemoved) {
                            ginst.LineState = RepPP.LineState.sdLineNew;
                        }
                        ginst.DeleteLine();
                    }                    
                }
                lv.Items.Remove(item);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAddClick:             Called when the add button is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************
        protected void OnAddClick(object sender, EventArgs e) {
            ListViewItem[]  items;
            
            items = new ListViewItem[m_lvAvailable.SelectedItems.Count];
            m_lvAvailable.SelectedItems.CopyTo(items, 0);
            CopyItemsToListView(m_lvAvailable, m_lvSelected, items);
            RemoveItemsFromListView(m_lvAvailable, m_bAvPhysRemoved, items);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnRemoveClick:          Called when the remove button is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************
        protected void OnRemoveClick(object sender, EventArgs e) {
            ListViewItem[]  items;
            
            items = new ListViewItem[m_lvSelected.SelectedItems.Count];
            m_lvSelected.SelectedItems.CopyTo(items, 0);
            CopyItemsToListView(m_lvSelected, m_lvAvailable, items);
            RemoveItemsFromListView(m_lvSelected, m_bSelPhysRemoved, items);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAvailableSelIndexChanged  Called when the selected index changed
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void OnAvailableSelIndexChanged(object sender, EventArgs e) {
		    m_butAdd.Enabled = (m_lvAvailable.SelectedItems.Count != 0);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSelectedSelIndexChanged   Called when the selected index changed
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        private void OnSelectedSelIndexChanged(object sender, EventArgs e) {
		    m_butRemove.Enabled = (m_lvSelected.SelectedItems.Count != 0);
        }
    }
}
