using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;

namespace RepPP.Toolkit.Window {

    //********************************************************************************
    //
    /// <summary>
    /// Base class for all forms which can serialize their display states.
    /// The base class handle the current form size, state and position.
    /// </summary>
    //
    //********************************************************************************    
    public class frmSerializabledForm : Form, RepPP.Toolkit.Window.IUIConfiguration {
        private System.ComponentModel.Container		components = null;
        private Point								m_ptsNorLocation;           // Form location when normal
        private Size								m_sizeNorSize;              // Form size when normal
        /// <summary>Configuration helper</summary>
        protected UIConfigurationHelper             m_configHelper;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                            Default constructor
        /// </summary>
        //
        //********************************************************************************
        public frmSerializabledForm() 
        {
            InitializeComponent();
        }

        //********************************************************************************
        //
        /// <summary>
        /// ConfigHelper:               Get and Set the configuration helper object
        /// </summary>
        //
        //********************************************************************************
        public virtual UIConfigurationHelper ConfigHelper 
        {
            get {
                return(m_configHelper);
            }
            set {
                m_configHelper = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DoNotLoadWindowState:        Flag indicating whether the
        ///                              Configuration Helper loads
        ///                              the window state or not
        /// </summary>
        /// <remarks>
        /// Setting the WindowState will force the display of the 
        /// form and may result in flikering while the form is 
        /// being resized.
        /// </remarks>
        //
        //*********************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool DoNotLoadWindowState {
            get {
                return (m_configHelper == null ? true : m_configHelper.DoNotLoadWindowState);
            }
            set {
                if (m_configHelper != null) {
                    m_configHelper.DoNotLoadWindowState = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Flush the form
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if called by the GC</param>
        //
        //********************************************************************************
        protected override void Dispose(bool bDisposing) 
        {
            if (bDisposing) {
                m_configHelper = null;
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLoad:                         Load the form state when form is loading
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            base.OnLoad(e);
            if (m_configHelper != null) {
                m_configHelper.LoadFormState(this);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnClosing:			        Called when the form is closing 
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //*********************************************************	    
        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if (!e.Cancel) {
                SaveConfiguration();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnFormPosSizeChanged:       Keep the last normal position and size of the form
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        public void OnFormPosSizeChanged(object sender, System.EventArgs e) 
        {
            if (WindowState == FormWindowState.Normal) {
                m_ptsNorLocation = Location;
                m_sizeNorSize    = Size;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadState:                      Load the form state.
        ///                                 Implementing RepPP.Toolkit.Window.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
        public virtual void LoadState(RepPP.Toolkit.Window.UIConfigurationHelper configHelper, XmlNode node) 
        {
            string                      strVersion;
            
            strVersion = configHelper.GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                configHelper.LoadFormPosState(this, node, out m_ptsNorLocation, out m_sizeNorSize);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveState:                      Save the form state
        ///                                 Implementing RepPP.Toolkit.Window.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
        public virtual void SaveState(UIConfigurationHelper configHelper, XmlNode node) 
        {
            node.RemoveAll();
            configHelper.SetNodeAttribute(node, "VERSION", "1");
            configHelper.SaveFormPosState(this, node, m_ptsNorLocation, m_sizeNorSize);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveConfiguration:				Save configuration of this form
        /// </summary>
        //
        //********************************************************************************
        public virtual void SaveConfiguration() 
        {
            if (m_configHelper != null) {
                m_configHelper.SaveFormState(this);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnFileSaveConfigClick:				Save UI Layout
        /// </summary>
        //
        //********************************************************************************
        protected virtual void OnFileSaveConfigClick() {
            try {
                SaveConfigurationToFile();
            } catch {
            }
        }

        //********************************************************************************
        //  
        /// <summary>
        /// SaveConfigurationToFile:			Save configuration to file 
        /// </summary>
        //  
        //********************************************************************************
        public void SaveConfigurationToFile() {
            SaveConfiguration();
            if (m_configHelper != null) {
                m_configHelper.SaveToFile();
            }
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.Size = new System.Drawing.Size(300,300);
            this.Text = "frmBaseForm";
            this.LocationChanged += new System.EventHandler(this.OnFormPosSizeChanged);
            this.Resize += new System.EventHandler(this.OnFormPosSizeChanged);
        }
        #endregion
    }/*frmBaseForm*/
}
